﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using System.DirectoryServices;
using System.DirectoryServices.AccountManagement;
using System.IO;

namespace CreateADUsers
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            //Sleep to let controls to be built
            System.Threading.Thread.Sleep(500);

            //Select the initial radio options
            rdoStaticPassword.Select();
            rdoSingleAccount.Select();
        }

        private void btnCreateUsers_Click(object sender, EventArgs e)
        {
            //**************************************************
            //Define local variables
            //**************************************************
            StreamWriter l_objWriter = null;
            string l_strMessage = "";
            string l_strErrorAccountList = "";
            int l_intSuccessAttempts = 0, l_intErrorAttempts = 0;

            try
            {
                //**************************************************
                //Grab values from form
                //**************************************************
                string l_strDomain = txtDomain.Text.Trim();
                string l_strDomainDN = txtDomainDN.Text.Trim();
                string l_strUserOU = txtUserOU.Text.Trim();
                string l_strUserPrefix = txtUserPrefix.Text.Trim();
                string l_strUsername = "", l_strPassword = "";
                int l_intQuantity = Convert.ToInt32(txtQuantity.Text.Trim());
                int l_intStartingIndex = Convert.ToInt32(txtStartingIndex.Text.Trim());
                int l_intPasswordLength = Convert.ToInt32(txtPasswordLength.Text.Trim());
                bool l_blnPasswordNeverExpires = chkPasswordNeverExpires.Checked;
                bool l_blnUserCannotChangePassword = chkUserCannotChangePassword.Checked;
                bool l_blnRequireCapital = chkRequireCapital.Checked;
                bool l_blnRequireLowerCase = chkRequireLowerCase.Checked;
                bool l_blnRequireNumber = chkRequireNumber.Checked;
                string l_strTextFileName = txtTextFileName.Text.Trim();
                string l_strTextFileLocation = txtTextFileLocation.Text.Trim();
                string l_strTextFileFullPath = l_strTextFileLocation + "\\" + l_strTextFileName;

                //******************************************************************
                //Check whether to create single account or multiple accounts
                //******************************************************************
                if (rdoSingleAccount.Checked)  //Create single account
                {
                    //Create user object
                    PrincipalContext l_objContext = new PrincipalContext(ContextType.Domain, l_strDomain, l_strUserOU);
                    UserPrincipal l_objNewUser = new UserPrincipal(l_objContext);

                    //Set username and password
                    l_strUsername = txtUsername.Text.Trim();
                    if (rdoStaticPassword.Checked) l_strPassword = txtPassword.Text.Trim();
                    else l_strPassword = this.GenerateRandomPassword(l_intPasswordLength, l_blnRequireCapital, l_blnRequireLowerCase, l_blnRequireNumber);
                    l_objNewUser.SamAccountName = l_strUsername;
                    l_objNewUser.SetPassword(l_strPassword);

                    //Assign user properties
                    l_objNewUser.UserPrincipalName = l_strUsername + "@" + l_strDomainDN;
                    l_objNewUser.DisplayName = l_strUsername;
                    l_objNewUser.Description = "Auto-generated account";
                    l_objNewUser.GivenName = l_strUsername;    //first name
                    //l_objNewUser.Surname = "";               //last name
                    l_objNewUser.PasswordNeverExpires = l_blnPasswordNeverExpires;
                    l_objNewUser.UserCannotChangePassword = l_blnUserCannotChangePassword;
                    l_objNewUser.Enabled = true;

                    //Check if user already exists within Active Directory
                    UserPrincipal usr = UserPrincipal.FindByIdentity(l_objContext, l_strUsername);
                    if (usr == null)  //User not found.  Proceed with save
                    {
                        //Save user to the directory
                        l_objNewUser.Save();
                        l_intSuccessAttempts++;
                    }
                    else //User already exists
                    {
                        if (l_strErrorAccountList == "") l_strErrorAccountList = l_strUsername;
                        else l_strErrorAccountList += ", " + l_strUsername;
                        l_intErrorAttempts++;
                    }

                    //Check if account details should be stored in text file
                    if (chkStoreInTextFile.Checked)
                    {
                        //Save account details to file
                        l_objWriter = new StreamWriter(l_strTextFileFullPath, false);
                        l_objWriter.WriteLine("1," + l_strUsername + "," + l_strPassword);
                        l_objWriter.Close();
                    }
                }
                else  //Create multiple accounts
                {
                    //Create the specified number of user accounts
                    for (int index = 1; index <= l_intQuantity; index++)
                    {
                        //Create user object
                        PrincipalContext l_objContext = new PrincipalContext(ContextType.Domain, l_strDomain, l_strUserOU);
                        UserPrincipal l_objNewUser = new UserPrincipal(l_objContext);

                        //Set username and password
                        l_strUsername = l_strUserPrefix + l_intStartingIndex.ToString();
                        if (rdoStaticPassword.Checked) l_strPassword = txtPassword.Text.Trim();
                        else l_strPassword = this.GenerateRandomPassword(l_intPasswordLength, l_blnRequireCapital, l_blnRequireLowerCase, l_blnRequireNumber);
                        l_objNewUser.SamAccountName = l_strUsername;
                        l_objNewUser.SetPassword(l_strPassword);

                        //Assign user properties
                        l_objNewUser.UserPrincipalName = l_strUsername + "@" + l_strDomainDN;
                        l_objNewUser.DisplayName = l_strUsername;
                        l_objNewUser.Description = "Auto-generated account";
                        l_objNewUser.GivenName = l_strUsername;    //first name
                        //l_objNewUser.Surname = "";               //last name
                        l_objNewUser.PasswordNeverExpires = l_blnPasswordNeverExpires;
                        l_objNewUser.UserCannotChangePassword = l_blnUserCannotChangePassword;
                        l_objNewUser.Enabled = true;

                        //Check if user already exists within Active Directory
                        UserPrincipal usr = UserPrincipal.FindByIdentity(l_objContext, l_strUsername);
                        if (usr == null)  //User not found.  Proceed with save
                        {
                            //Save user to the directory
                            l_objNewUser.Save();
                            l_intSuccessAttempts++;

                            //Check if account details should be stored in text file
                            if (chkStoreInTextFile.Checked)
                            {
                                //Save account details to file
                                if (l_objWriter == null) l_objWriter = new StreamWriter(l_strTextFileFullPath, false);
                                l_objWriter.WriteLine(index + "," + l_strUsername + "," + l_strPassword);
                            }
                        }
                        else //User already exists
                        {
                            if (l_strErrorAccountList == "") l_strErrorAccountList = l_strUsername;
                            else l_strErrorAccountList += ", " + l_strUsername;
                            l_intErrorAttempts++;
                        }

                        //Increment starting index
                        l_intStartingIndex++;

                        //Close writer if this was the last check
                        if (chkStoreInTextFile.Checked && index == l_intQuantity && l_objWriter != null) l_objWriter.Close();
                    }
                }

                //Write event log entry and display status to user
                if (l_intSuccessAttempts > 0)
                {
                    l_strMessage += "Successfully created " + l_intSuccessAttempts.ToString() + " account(s).\r\n";
                }
                if (l_intErrorAttempts > 0)
                {
                    l_strMessage += "Could not create " + l_intErrorAttempts.ToString() + " account(s): " + l_strErrorAccountList + "\r\n\r\n";
                    l_strMessage += "If any accounts were not created, they most likely already exist within Active Directory.";
                }
                System.Diagnostics.EventLog.WriteEntry("CreateADUsers", l_strMessage, EventLogEntryType.Information, 1001);
                MessageBox.Show("Task Completed!\r\n" + l_strMessage, "Create Users Task");
            }
            catch (Exception objException)
            {
                //Write event log entry
                System.Diagnostics.EventLog.WriteEntry("CreateADUsers", "Error creating user within application.\r\n\r\nAdditional details:\r\n" + objException.Message, EventLogEntryType.Error, 1002);
                MessageBox.Show("Error!  Please check event viewer...", "Create Users Task");
            }
        }

        public string GenerateRandomPassword(int p_intPasswordLength, bool p_blnRequireCapital, bool p_blnRequireLowerCase, bool p_blnRequireNumber)
        {
            //Define available characters for password
            string[] l_strCapitals = { "A","B","C","D","E","F","G","H","I","J","K","L","M","N","O","P","Q","R","S","T","U","V","W","X","Y","Z" };
            string[] l_strLowerCases = { "a","b","c","d","e","f","g","h","i","j","k","l","m","n","o","p","q","r","s","t","u","v","w","x","y","z" };
            string[] l_strNumbers = { "1","2","3","4","5","6","7","8","9","0" };
            string[] l_strCharacters = { "A","B","C","D","E","F","G",
                                         "H","I","J","K","L","M","N","O","P","Q","R","S","T","U","V","W","X","Y",
                                         "Z","1","2","3","4","5","6","7","8","9","0","a","b","c","d","e","f","g","h",
                                         "i","j","k","l","m","n","o","p","q","r","s","t","u","v","w","x","y","z"};

            //Define additional variables
            string l_strPassword = "";
            Random rGen = new Random();
            int l_intPosition = 0;

            //Check if capital is required...
            if (p_blnRequireCapital && p_intPasswordLength > 0)
            {
                l_intPosition = rGen.Next(0, 25);
                l_strPassword += l_strCapitals[l_intPosition];
                p_intPasswordLength = p_intPasswordLength - 1;  //Adjust remaining password length
            }

            //Check if lower case is required...
            if (p_blnRequireLowerCase && p_intPasswordLength > 0)
            {
                l_intPosition = rGen.Next(0, 25);
                l_strPassword += l_strLowerCases[l_intPosition];
                p_intPasswordLength = p_intPasswordLength - 1;  //Adjust remaining password length
            }

            //Check if number is required...
            if (p_blnRequireNumber && p_intPasswordLength > 0)
            {
                l_intPosition = rGen.Next(0, 9);
                l_strPassword += l_strNumbers[l_intPosition];
                p_intPasswordLength = p_intPasswordLength - 1;  //Adjust remaining password length
            }

            //Generate remaining password characters
            for (int i = 1; i <= p_intPasswordLength; i++)
            {
                l_intPosition = rGen.Next(0, 60);
                l_strPassword += l_strCharacters[l_intPosition];
            }

            //Return password
            return l_strPassword;
        }

        private void rdoSingleAccount_CheckedChanged(object sender, EventArgs e)
        {
            //Enable/disable controls
            lblUsername.Enabled = true;
            lblUserPrefix.Enabled = false;
            lblStartingIndex.Enabled = false;
            lblQuantity.Enabled = false;
            txtUsername.Enabled = true;
            txtUserPrefix.Enabled = false;
            txtStartingIndex.Enabled = false;
            txtQuantity.Enabled = false;
        }

        private void rdoMultipleAccounts_CheckedChanged(object sender, EventArgs e)
        {
            //Enable/disable controls
            lblUsername.Enabled = false;
            lblUserPrefix.Enabled = true;
            lblStartingIndex.Enabled = true;
            lblQuantity.Enabled = true;
            txtUsername.Enabled = false;
            txtUserPrefix.Enabled = true;
            txtStartingIndex.Enabled = true;
            txtQuantity.Enabled = true;
        }

        private void rdoStaticPassword_CheckedChanged(object sender, EventArgs e)
        {
            lblPassword.Enabled = true;
            lblPasswordLength.Enabled = false;
            lblRequireCapital.Enabled = false;
            lblRequireLowerCase.Enabled = false;
            lblRequireNumber.Enabled = false;
            txtPassword.Enabled = true;
            txtPasswordLength.Enabled = false;
            chkRequireCapital.Enabled = false;
            chkRequireLowerCase.Enabled = false;
            chkRequireNumber.Enabled = false;
        }

        private void rdoRandomPassword_CheckedChanged(object sender, EventArgs e)
        {
            lblPassword.Enabled = false;
            lblPasswordLength.Enabled = true;
            lblRequireCapital.Enabled = true;
            lblRequireLowerCase.Enabled = true;
            lblRequireNumber.Enabled = true;
            txtPassword.Enabled = false;
            txtPasswordLength.Enabled = true;
            chkRequireCapital.Enabled = true;
            chkRequireLowerCase.Enabled = true;
            chkRequireNumber.Enabled = true;
        }

        private void chkStoreInTextFile_CheckedChanged(object sender, EventArgs e)
        {
            if (chkStoreInTextFile.Checked)
            {
                lblTextFileName.Enabled = true;
                txtTextFileName.Enabled = true;
                lblTextFileLocation.Enabled = true;
                txtTextFileLocation.Enabled = true;
                btnBrowse.Enabled = true;
            }
            else
            {
                lblTextFileName.Enabled = false;
                txtTextFileName.Enabled = false;
                lblTextFileLocation.Enabled = false;
                txtTextFileLocation.Enabled = false;
                btnBrowse.Enabled = false;
            }
        }

        private void btnBrowse_Click(object sender, EventArgs e)
        {
            DialogResult result = this.folderBrowserDialog1.ShowDialog();
            if (result == DialogResult.OK) //User selected "Open" within dialog
            {
                txtTextFileLocation.Text = this.folderBrowserDialog1.SelectedPath;
            }
        }

        //public void OldWay()
        //{
        //    string l_strDebugInfo = "";
        //    try
        //    {
        //        string l_strLDAPPath = txtLDAPPath.Text.Trim();
        //        string l_strUserID = txtUsername.Text.Trim();
        //        string l_strPassword = txtPassword.Text.Trim();
        //        string l_strDomain = txtDomain.Text.Trim();

        //        //Create account
        //        l_strDebugInfo += "Creating account...";
        //        DirectoryEntry l_objDirectoryEntry = new DirectoryEntry(l_strLDAPPath);
        //        DirectoryEntries l_objChildren = l_objDirectoryEntry.Children;
        //        DirectoryEntry l_objNewUser = l_objChildren.Add("CN=" + l_strUserID, "user");
        //        l_strDebugInfo += "Account created successfully.\r\n";

        //        //Set properties
        //        l_strDebugInfo += "Setting properties...";
        //        l_objNewUser.Properties["sAMAccountName"].Value = l_strUserID;
        //        l_objNewUser.Properties["userPrincipalName"].Value = l_strUserID + "@" + l_strDomain;
        //        l_objNewUser.Properties["givenName"].Value = "First";
        //        l_objNewUser.Properties["sn"].Value = "Last";
        //        l_objNewUser.Properties["DisplayName"].Value = l_strUserID;
        //        l_objNewUser.Properties["description"].Value = "Auto-generated account";
        //        l_objNewUser.CommitChanges();
        //        l_strDebugInfo += "Properties set successfully.\r\n";

        //        //Set password
        //        l_strDebugInfo += "Setting password...";
        //        object[] l_objPassword = new object[] { l_strPassword };
        //        object l_objReturn = l_objNewUser.Invoke("SetPassword", l_objPassword);
        //        l_objNewUser.CommitChanges();
        //        l_strDebugInfo += "Password set successfully.\r\n";

        //        //Set attributes on account
        //        l_strDebugInfo += "Setting password attributes...";
        //        int l_intVal = 0;
        //        //l_intVal = (int)l_objNewUser.Properties["userAccountControl"].Value;
        //        //l_objNewUser.Properties["userAccountControl"].Value = l_intVal | 0x0040;  //password can't change 
        //        //l_objNewUser.CommitChanges();
        //        l_intVal = (int)l_objNewUser.Properties["userAccountControl"].Value;
        //        l_objNewUser.Properties["userAccountControl"].Value = l_intVal | 0x10000; //don't expire password
        //        l_objNewUser.CommitChanges();
        //        l_intVal = (int)l_objNewUser.Properties["userAccountControl"].Value;
        //        l_objNewUser.Properties["userAccountControl"].Value = l_intVal & ~0x0002; //enable account
        //        //l_objNewUser.Properties["userAccountControl"].Value = l_intVal & 0x0002; //disable account               
        //        l_objNewUser.CommitChanges();
        //        l_strDebugInfo += "Attributes set successfully.\r\n";

        //        //Close objects
        //        l_strDebugInfo += "Closing objects...";
        //        l_objNewUser.Close();
        //        l_objDirectoryEntry.Close();
        //        l_strDebugInfo += "Closed objects successfully.\r\n";

        //        //Write event log entry
        //        System.Diagnostics.EventLog.WriteEntry("CreateADUsers", "Successfully created user '" + l_strUserID + "'.  \r\n\r\nExecution Details:\r\n" + l_strDebugInfo, EventLogEntryType.Information, 1001);

        //        MessageBox.Show("Done!", "Create Users Task");
        //    }
        //    catch (Exception objException)
        //    {
        //        //Write event log entry
        //        System.Diagnostics.EventLog.WriteEntry("CreateADUsers", "Error creating user within application.  \r\n\r\nExecution Details:\r\n" + l_strDebugInfo + "\r\n\r\nAdditional details: \r\n" + objException.Message, EventLogEntryType.Error, 1002);

        //        MessageBox.Show("Error!  Please check event viewer...", "Create Users Task");
        //    }
        //}
    }
}
